<?php

class CfpWebTemplates {
    public $vars = array();
    public $templatePath;
    public $templateResult;
    public $parentTemplate;

    public function __construct() {
    }

    public function put($name, $value, $encode=true) {
        if(!is_string($name)) {
            throw new BadMethodCallException('First argument for '.__METHOD__.'() must be of type string, '.gettype($name).' given.');
        }
        if($encode && !is_object($value)) {
            $value = htmlentities($value);
        }
        $this->vars[strtoupper($name)] = $value;
    }

    public function putArray(array $variables) {
        foreach($variables as $name => $value) {
            if(is_array($value)) {
                $encode = (bool)$value[1];
                $value = $value[0];
            }
            $this->put($name, $value);
        }
    }

    public function remove($name) {
        if(!is_string($name)) {
            throw new BadMethodCallException('First argument for '.__METHOD__.'() must be of type string, '.gettype($name).' given.');
        }
        if(isset($this->vars[strtoupper($name)])) {
            unset($this->vars[strtoupper($name)]);
        }
    }

    public function removeArray(array $names) {
        foreach($names as $name) {
            $this->remove($name);
        }
    }

    public function setParent(CfpWebTemplates $parent) {
        $this->parentTemplate = $parent;
    }

    public function removeParent() {
        $this->parentTemplate = null;
    }

    public function setTemplatePath($path) {
        if(!file_exists($path)) {
            throw new CfpIoException('Unable to load template: file does not exist.');
        }
        $this->templatePath = $path;
    }

    public function isRendered() {
        return !empty($this->templateResult);
    }

    public function varsMergeParent($vars) {
        if(is_a($this->parentTemplate, __CLASS__)) {
            $vars = array_merge($this->parentTemplate->vars, $vars);
            $vars = $this->parentTemplate->varsMergeParent($vars);
        }
        return is_array($vars) ? $vars : array();
    }

    public function render($path=null) {
        if($path) {
            $this->setTemplatePath($path);
        }
        if(empty($this->templatePath)) {
            throw new RuntimeException(__METHOD__.' requires template path to be set, template path is empty.');
        }
        $template = file_get_contents($this->templatePath);
        $vars = $this->varsMergeParent($this->vars);
        foreach($vars as $name => $value) {
            if(is_a($value, __CLASS__)) {
                if(!$value->isRendered()) {
                    $value->render();
                }
                $value = $value->fetchResult();
            }
            $template = str_replace('{'.$name.'}', $value, $template);
        }
        $this->templateResult .= $template;
    }

    public function displayResult() {
        print $this->templateResult;
    }

    public function clearResult() {
        $this->templateResult = null;
    }

    public function flushResult() {
        $this->displayResult();
        $this->clearResult();
    }

    public function fetchResult() {
        return $this->templateResult;
    }
}

?>