<?php
/**
 * Utility for Deployment of CairoForPHP
 *
 * This script will allow for the deployment of the source tree in different
 * setups from plain copies to phar archives and compression of deployment
 * packages etc.
 *
 * PHP version 5.3.x
 *
 * LICENSE: This source file is subject to version 3.01 of the PHP license
 * that is available through the world-wide-web at the following URI:
 * http://www.php.net/license/3_01.txt.  If you did not receive a copy of
 * the PHP License and are unable to obtain it through the web, please
 * send a note to license@php.net so we can mail you a copy immediately.
 *
 * @package    CairoForPHP
 * @subpackage Utilities
 * @author     Florian F Freeman <florian@phpws.org>
 * @copyright  2009 Florian F Freeman
 * @license    http://www.php.net/license/3_01.txt  PHP License 3.01
 * @version    CVS: $Id:$
 * @link       http://gtk.php.net
 * @since      1.0.0
 */
//Auxiliary functions
require_once(dirname(__FILE__).'/utilFuncs.php');

//Check if option is help...
if($_SERVER['argv'][1] == '--help') {
    goto makeHelp;
}

//Print a message to acknowledge deployment..
print "Beginning deploymend procedure...\n\n";

//Find directories..
$prj_dir = dirname(dirname(__FILE__));
$src_dir = $prj_dir.'/src';
$dply_dir = $prj_dir.'/deploy';

//Clean deploy directory..
if(!array_search('clean=0', $_SERVER['argv'])) {
    print "Cleaning deployment directory...\n";
    rrmdir($dply_dir);
    mkdir($dply_dir);
    print "  Done.\n";
}

//Reset cfp-conf.ini
print "Restoring defaultConf.ini to cfp-conf.ini...\n";
if(file_exists($src_dir.'/cfp-conf.ini')) {
    unlink($src_dir.'/cfp-conf.ini');
}
copy($src_dir.'/res/defaultConf.ini', $src_dir.'/cfp-conf.ini');
print "  Done.\n";

//Go over to correct procedure..
switch($_SERVER['argv'][1]) {
  case '--plain':
    goto makePlain;
    break;
  case '--phar':
    goto makePhar;
    break;
  case '--bcomp':
    goto makeBcomp;
    break;
  case '--clean':
    goto EOC;
    break;
  default:
    goto makeInvalid;
}

makePlain:
print "Building plain copy...\n";
mkdir($dply_dir.'/plain');
if( rcopy($src_dir, $dply_dir.'/plain') )
  print "  Done.";
else
  print "! Failed: failed to copy directories.\n";
goto EOC;

makePhar:
print "Building Phar archive...\n";
mkdir($dply_dir.'/phar');
$phar = new Phar($dply_dir.'/phar/cfp.phar');
$phar->buildFromDirectory($src_dir);
$phar_stub = '<?php
if( !class_exists("Phar") ) {
  print "This program requires PHP to have Phar support enabled.\n";
  print "Class ``Phar\'\' could not be found.\n";
  exit(1);
}
Phar::mapPhar();
Phar::interceptFileFuncs();
define("CFP_IS_PHAR", true);

include("phar://".__FILE__."/main.php");
__HALT_COMPILER();';
$phar->setStub($phar_stub);
print "  Done.\n";

if(array_search('gz=1', $_SERVER['argv'])) {
    print "Compressing Phar archive with zlib (gz)...\n";
    if( $phar->canCompress(Phar::GZ) ) {
      $phar->compress(Phar::GZ);
      //Fix the Bootstrap!
      $comp_phar = new Phar($dply_dir.'/phar/cfp.phar.gz');
      $comp_phar->setStub($phar_stub);
      unset($comp_phar);
      print "  Done.\n";
    } else {
      print "  Failed: zlib compression not available.\n";
    }
}
if(array_search('bz=1', $_SERVER['argv'])) {
    print "Compressing Phar archive with bzip2 (bz2)...\n";
    if( $phar->canCompress(Phar::BZ2) ) {
      $phar->compress(Phar::BZ2);
      //Fix the Bootstrap!
      $comp_phar = new Phar($dply_dir.'/phar/cfp.phar.bz2');
      $comp_phar->setStub($phar_stub);
      unset($comp_phar);
      print "  Done.\n";
    } else {
      print "! Failed: bzip2 compression not available.\n";
    }
}

unset($phar);
goto EOC;

makeBcomp:
print "Building bytecode compiled copy...\n";
print "! Failed: method not implemented.\n";
goto EOC;

makeHelp:
print "Usage: deploy[.php] <mode> [<compression>] [<option>]\n\n";
print "Modes:\n";
print "               This specifies the mode in which the deployment script\n";
print "               is operating. You can only choose one mode at a time,\n";
print "               duplicate specifications will be ignored.\n\n";
print "  --clean      Just clean up the deployment directory.\n";
print "  --plain      Make a plain copy of source tree, ready to use.\n";
print "  --phar       Create a single phar archive from source tree.\n";
print "  --bcomp      Use bcompiler to precompile all php files,\n";
print "               otherwise behaving like --plain\n";
print "  --help       Show this help.\n\n";
print "Compression:\n";
print "               All compression methods are switched off by default.\n";
print "               Multiple compression methods may be specified.\n\n";
print "  gz=(0|1)     Switch Zlib compression of resulting package on/off.\n";
print "  bz=(0|1)     Switch BZip compression of resulting package on/off.\n\n";
print "Options:\n";
print "               These are additional options that affect the behaviour\n";
print "               of this script. Multiple options may be specified.\n\n";
print "  clean=(0|1)  Whether or not to clean the deployment directory before\n";
print "               actually deploying anything. Default is on.\n";
goto EOF;

makeInvalid:
print "Unknown procedure...\n";
print "! The specified procedure ``".$_SERVER['argv'][1]."'' does not exist.\n";
print "! Please check the --help option for available procedures.\n";
goto EOC;

EOC:
//Let user know we have finished...
print "\nEnd of deployment procedure.\n";

EOF:
?>